==> 0.0.1.tar.gz: android/src/main/kotlin/io/stratosfy/esp_bluetooth_provisioning/EspBluetoothProvisioningPlugin.kt <==
package io.stratosfy.esp_bluetooth_provisioning

import android.Manifest
import android.bluetooth.BluetoothDevice
import android.bluetooth.le.ScanResult
import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import android.os.Handler
import android.util.Log
import androidx.annotation.NonNull
import androidx.annotation.RequiresApi
import androidx.core.app.ActivityCompat
import com.espressif.provisioning.ESPConstants
import com.espressif.provisioning.ESPProvisionManager
import com.espressif.provisioning.device_scanner.BleScanner
import com.espressif.provisioning.listeners.BleScanListener
import com.espressif.provisioning.listeners.ProvisionListener

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.stratosfy.esp_ble_provisioning.BleDevice
import io.stratosfy.esp_ble_provisioning.PermissionManager

/** EspBluetoothProvisioningPlugin */
class EspBluetoothProvisioningPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  private lateinit var channel : MethodChannel
  private lateinit var context: Context
  private var bleScanner: BleScanner? = null
  private val deviceNamePrefix: String? = null
  private lateinit var permissionManager: PermissionManager
  private lateinit var provisionManager: ESPProvisionManager
  private var devices = ArrayList<BleDevice>()
  private var provisionStatus : String? = null
  private var ssid : String? = null
  private var password : String? = null



  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "esp_ble_provisioning")
    channel.setMethodCallHandler(this)
  }

  private fun initilizeWithContext() {
    permissionManager = PermissionManager(context)
    provisionManager = ESPProvisionManager.getInstance(context)
  }

  @RequiresApi(Build.VERSION_CODES.O)
  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    val method: String = call.method
    when {
      method.contains("startProvisioningFlow") -> {
//        val uuids: ArrayList<String> = call.argument("devices")!!

        Log.e("BLEProvisionLanding", "Starting provisioning")

        if (ActivityCompat.checkSelfPermission(context, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED
                && ActivityCompat.checkSelfPermission(context, Manifest.permission.BLUETOOTH) == PackageManager.PERMISSION_GRANTED
                && ActivityCompat.checkSelfPermission(context, Manifest.permission.BLUETOOTH_ADMIN) == PackageManager.PERMISSION_GRANTED)

        {
          Log.e("BLEProvisionLanding", "Searching for devices")
          provisionManager = ESPProvisionManager.getInstance(context)
//            devices.clear()
          provisionManager.searchBleEspDevices(deviceNamePrefix, bleScanListener)
        }
        else
        {
          Log.e("BLEProvisionLanding", "Not able to start scan as Location permission is not granted. Failed!")
        }

        Handler().postDelayed({
          Log.d("BLEProvisionLanding", "DEVICES : $devices")
          val map = devices.associateBy({it.name}, {it.uuid})
          Log.d("BLEProvisionLanding", "DEVICES MAP: $map")

          result.success(map)

        }, 5000)

      }

      method.contains("ConnectToBlueTooth") -> {
        Log.e("BLEProvisionLanding", "Connect to device")
        val name: String = call.argument("name")!!

        provisionManager.createESPDevice(ESPConstants.TransportType.TRANSPORT_BLE, ESPConstants.SecurityType.SECURITY_1)

        var device = BleDevice()
        var i = 0
        if(i < devices.size)
        {
          if(name.equals(devices[i].name))
          {
            device = devices[i]
          }
        }

        Log.d("BLEProvisionLanding", "=================== Connect to device : ${device.bluetoothDevice}")
        Log.d("BLEProvisionLanding", "=================== Connect to device : " + device.name + " UUID : " + device.uuid)

        if (ActivityCompat.checkSelfPermission(context, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED)
        {
          Log.e("BLEProvisionLanding", "Before Connecting ------------------>.")
          var res = provisionManager?.espDevice?.connectBLEDevice(device.bluetoothDevice, device.uuid)
//                    handler.postDelayed(disconnectDeviceTask, 20000)

//                    var res = provisionManager?.getEspDevice()?.connectBLEDevice(bleDevice.bluetoothDevice, uuid)
          Log.d("BLEProvisionLanding", "=================== Connected to device : " + res)


          Log.e("BLEProvisionLanding", "After Connecting ------------------>.")

          result.success(true)

        }
        else
        {
          result.error("400", "Failed to connect to device", "Error connecting to device")
        }

      }

      method == "StartProvisioning" -> {

        ssid = call.argument("ssid")!!
        password = call.argument("password")!!


        Handler().postDelayed({

          Log.e("BLEProvisionLanding", "Before Security check ------------------>.")
          var sec = provisionManager.espDevice?.setProofOfPossession("abcd1234")
          Log.e("BLEProvisionLanding", "After Security check ------------------>.")

          //**********PROVISIONING PART

          Handler().postDelayed({
            val status = doProvisioning(ssid!!, password!!)
            Log.d("BLEProvisionLanding", "Device Provisioned -------------------->")
            result.success(status)
          }, 5000)


        }, 5000)
      }

      method == "permissionState" -> {
        result.success(permissionManager.bluetoothPermissionState)
      }
      method == "bluetoothState" -> {
        result.success(permissionManager.bluetoothState)
      }
      method.contains("bluetoothStateListener") -> {
        permissionManager.bluetoothState(method, object : PermissionManager.BluetoothStateListener {

          override fun onStateChanged(bluetoothState: PermissionManager.BluetoothState) {
//            sendBluetoothResponse(method, bluetoothState.state)
          }
        })
        result.success(true)
      }
    }
  }


  private fun doProvisioning(ssid: String, password: String) : String? {

    provisionManager.espDevice.provision(ssid, password, object : ProvisionListener {
      override fun createSessionFailed(e: java.lang.Exception) {
        Log.e("BLEProvisionLanding", "createSessionFailed!!")
      }

      override fun wifiConfigSent() {
        Log.e("BLEProvisionLanding", "wifiConfigSent!!")
        provisionStatus = "wifiConfigSent!!"

      }

      override fun wifiConfigFailed(e: java.lang.Exception) {
        Log.e("BLEProvisionLanding", "wifiConfigFailed!!")

      }

      override fun wifiConfigApplied() {
        Log.e("BLEProvisionLanding", "wifiConfigApplied!!")
        provisionStatus = "wifiConfigApplied!!"
      }

      override fun wifiConfigApplyFailed(e: java.lang.Exception) {
        Log.e("BLEProvisionLanding", "wifiConfigApplyFailed!!")
      }

      override fun provisioningFailedFromDevice(failureReason: ESPConstants.ProvisionFailureReason) {

      }

      override fun deviceProvisioningSuccess() {
        Log.e("BLEProvisionLanding", "deviceProvisioningSuccess!!")
        provisionStatus = "Provisioned!!"
      }

      override fun onProvisioningFailed(e: java.lang.Exception) {
        Log.e("BLEProvisionLanding", "onProvisioningFailed!!")
      }

    })

    return provisionStatus
  }

  private val bleScanListener: BleScanListener = object : BleScanListener {

    override fun scanStartFailed() {
      Log.d("BLEProvisionLanding", "Please turn on Bluetooth to connect BLE device")

//            Toast.makeText(this, "Please turn on Bluetooth to connect BLE device", Toast.LENGTH_SHORT).show()
    }

    override fun onPeripheralFound(device: BluetoothDevice, scanResult: ScanResult) {
      Log.d("BLEProvisionLanding", "BLE device $device")
      val dev = BleDevice()
      dev.name = scanResult.scanRecord!!.deviceName.toString()
      if (scanResult.scanRecord!!.serviceUuids != null && scanResult.scanRecord!!.serviceUuids.size > 0) {
        dev.uuid = scanResult.scanRecord!!.serviceUuids[0].toString()
      }
      dev.bluetoothDevice = (device)

      if(!devices.contains(dev))
      {
        devices.add(dev)
      }
    }

    override fun scanCompleted() {
      Log.d("BLEProvisionLanding", "Scan completed")

//            isScanning = false
//            updateProgressAndScanBtn()
    }

    override fun onFailure(e: Exception) {
      Log.e("BLEProvisionLanding", e.message!!)
      e.printStackTrace()
    }
  }


  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    this.context = binding.activity
    initilizeWithContext()
  }

  override fun onDetachedFromActivityForConfigChanges() {
    TODO("Not yet implemented")
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    TODO("Not yet implemented")
  }

  override fun onDetachedFromActivity() {
    TODO("Not yet implemented")
  }
}

==> 0.0.2.tar.gz: android/src/main/kotlin/io/stratosfy/esp_bluetooth_provisioning/EspBluetoothProvisioningPlugin.kt <==
package io.stratosfy.esp_bluetooth_provisioning

import android.Manifest
import android.bluetooth.BluetoothDevice
import android.bluetooth.le.ScanResult
import android.content.Context
import android.content.SharedPreferences
import android.content.pm.PackageManager
import android.os.Build
import android.os.Handler
import android.util.Log
import androidx.annotation.NonNull
import androidx.annotation.RequiresApi
import androidx.core.app.ActivityCompat
import com.espressif.provisioning.ESPConstants
import com.espressif.provisioning.ESPProvisionManager
import com.espressif.provisioning.device_scanner.BleScanner
import com.espressif.provisioning.listeners.BleScanListener
import com.espressif.provisioning.listeners.ProvisionListener
import com.google.gson.Gson
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.stratosfy.esp_ble_provisioning.BleDevice
import io.stratosfy.esp_ble_provisioning.PermissionManager


/** EspBluetoothProvisioningPlugin */
class EspBluetoothProvisioningPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  private lateinit var channel : MethodChannel
  private lateinit var context: Context
  private var bleScanner: BleScanner? = null
  private val deviceNamePrefix: String? = null
  private lateinit var permissionManager: PermissionManager
  private lateinit var provisionManager: ESPProvisionManager
  private lateinit var sharedPreferences: SharedPreferences
  private var devices = ArrayList<BleDevice>()
  private var provisionStatus : String? = null
  private var ssid : String? = null
  private var password : String? = null
  private var selectedDevice : BleDevice? = null
  private var selectedDeviceName : String? = null




  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "esp_ble_provisioning")
    channel.setMethodCallHandler(this)
  }

  private fun initilizeWithContext() {
    permissionManager = PermissionManager(context)
    provisionManager = ESPProvisionManager.getInstance(context)
    sharedPreferences = context.getSharedPreferences("esp_ble_provisioning", Context.MODE_PRIVATE)

  }

  @Suppress("NULLABILITY_MISMATCH_BASED_ON_JAVA_ANNOTATIONS")
  @RequiresApi(Build.VERSION_CODES.O)
  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    val method: String = call.method
    when {
      method.contains("startProvisioningFlow") -> {
//        val uuids: ArrayList<String> = call.argument("devices")!!

        Log.e("BLEProvisionLanding", "Starting provisioning")

        if (ActivityCompat.checkSelfPermission(context, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED
                && ActivityCompat.checkSelfPermission(context, Manifest.permission.BLUETOOTH) == PackageManager.PERMISSION_GRANTED
                && ActivityCompat.checkSelfPermission(context, Manifest.permission.BLUETOOTH_ADMIN) == PackageManager.PERMISSION_GRANTED)

        {
          Log.e("BLEProvisionLanding", "Searching for devices")
          provisionManager = ESPProvisionManager.getInstance(context)
//            devices.clear()
          provisionManager.searchBleEspDevices(deviceNamePrefix, bleScanListener)
        }
        else
        {
          Log.e("BLEProvisionLanding", "Not able to start scan as Location permission is not granted. Failed!")
        }

        Handler().postDelayed({
          Log.d("BLEProvisionLanding", "DEVICES : $devices")
          val map = devices.associateBy({ it.name }, { it.uuid })
          Log.d("BLEProvisionLanding", "DEVICES MAP: $map")

          val gson = Gson()
          val i = 0
          val set: MutableSet<String> = HashSet()
          if (i < devices.size) {
            val json: String = gson.toJson(devices[i])
            set.add(json)
          }

          val bef = sharedPreferences.edit().putStringSet("devices", set).apply()
          Log.d("BLEProvisionLanding", "STORED SHARED PREFERENCE : ${set}")



          result.success(map) }, 5000)

      }

      method.contains("ConnectToBlueTooth") -> {
        selectedDeviceName = call.argument("name")!!

        Log.e("BLEProvisionLanding", "Connect to device $selectedDeviceName")

//        val share: Set<String>? = sharedPreferences.getStringSet("devices", null)
//
//        Log.d("BLEProvisionLanding", "SHARED PREFERENCE : $share")
//
//
//        val j = 0
//        val gson = Gson()
//
//        devices.clear()
//        val list: List<String> = ArrayList<String>(share)
//        if (j < list.size) {
////          val productNameList = list[j]['bluetoothDevice']
//
//          val mObject: BleDevice = gson.fromJson(list[j], BleDevice::class.java)
////          val obj: BluetoothDevice = gson.fromJson(list[j].ge, BluetoothDevice::class.java)
//
//          Log.d("BLEProvisionLanding", "SHARED PREFERENCE : ${mObject.bluetoothDevice} + ${mObject.name} ")
//          devices.add(mObject)
//        }
//
//        Log.d("BLEProvisionLanding", "SHARED PREFERENCE : $devices")

        devices.clear()
        if (ActivityCompat.checkSelfPermission(context, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED
                && ActivityCompat.checkSelfPermission(context, Manifest.permission.BLUETOOTH) == PackageManager.PERMISSION_GRANTED
                && ActivityCompat.checkSelfPermission(context, Manifest.permission.BLUETOOTH_ADMIN) == PackageManager.PERMISSION_GRANTED)

        {
          Log.e("BLEProvisionLanding", "Searching for devices")
          provisionManager = ESPProvisionManager.getInstance(context)
//            devices.clear()
          provisionManager.searchBleEspDevices(deviceNamePrefix, bleScanListener)
        }
        else
        {
          Log.e("BLEProvisionLanding", "Not able to start scan as Location permission is not granted. Failed!")
        }

        Handler().postDelayed({
          provisionManager.createESPDevice(ESPConstants.TransportType.TRANSPORT_BLE, ESPConstants.SecurityType.SECURITY_1)

//          var device = BleDevice()
//          var i = 0
//          if(i < devices.size)
//          {
//            if(name.equals(devices[i].name))
//            {
//              Log.e("BLEProvisionLanding", "Selected device ------------------>" + devices[i].bluetoothDevice);
//              device = devices[i]
//            }
//          }

          Log.d("BLEProvisionLanding", "=================== Connect to device : ${selectedDevice?.bluetoothDevice}")
          Log.d("BLEProvisionLanding", "=================== Connect to device : " + selectedDevice?.name + " UUID : " + selectedDevice?.uuid)

          if (ActivityCompat.checkSelfPermission(context, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED)
          {
            Log.e("BLEProvisionLanding", "Before Connecting ------------------>.")
            provisionManager.espDevice?.connectBLEDevice(selectedDevice?.bluetoothDevice, selectedDevice?.uuid)
//                    handler.postDelayed(disconnectDeviceTask, 20000)

//                    var res = provisionManager?.getEspDevice()?.connectBLEDevice(bleDevice.bluetoothDevice, uuid)
            Log.d("BLEProvisionLanding", "=================== Connected to device : ")


            Log.e("BLEProvisionLanding", "After Connecting ------------------>.")

            result.success(true)

          }
          else
          {
            result.error("400", "Failed to connect to device", "Error connecting to device")
          }

        }, 5000)


      }

      method == "StartProvisioning" -> {

        ssid = call.argument("ssid")!!
        password = call.argument("password")!!


        Handler().postDelayed({

          Log.e("BLEProvisionLanding", "Before Security check ------------------>.")
          var sec = provisionManager.espDevice?.setProofOfPossession("abcd1234")
          Log.e("BLEProvisionLanding", "After Security check ------------------>.")

          //**********PROVISIONING PART

          Handler().postDelayed({
            val status = doProvisioning(ssid!!, password!!)
            Log.d("BLEProvisionLanding", "Device Provisioned -------------------->")
            result.success(status)
          }, 5000)


        }, 5000)
      }

      method == "permissionState" -> {
        result.success(permissionManager.bluetoothPermissionState)
      }
      method == "bluetoothState" -> {
        result.success(permissionManager.bluetoothState)
      }
      method.contains("bluetoothStateListener") -> {
        permissionManager.bluetoothState(method, object : PermissionManager.BluetoothStateListener {

          override fun onStateChanged(bluetoothState: PermissionManager.BluetoothState) {
//            sendBluetoothResponse(method, bluetoothState.state)
          }
        })
        result.success(true)
      }
    }
  }


  private fun doProvisioning(ssid: String, password: String) : String? {

    provisionManager.espDevice.provision(ssid, password, object : ProvisionListener {
      override fun createSessionFailed(e: java.lang.Exception) {
        Log.e("BLEProvisionLanding", "createSessionFailed!!")
      }

      override fun wifiConfigSent() {
        Log.e("BLEProvisionLanding", "wifiConfigSent!!")
        provisionStatus = "wifiConfigSent!!"

      }

      override fun wifiConfigFailed(e: java.lang.Exception) {
        Log.e("BLEProvisionLanding", "wifiConfigFailed!!")

      }

      override fun wifiConfigApplied() {
        Log.e("BLEProvisionLanding", "wifiConfigApplied!!")
        provisionStatus = "wifiConfigApplied!!"
      }

      override fun wifiConfigApplyFailed(e: java.lang.Exception) {
        Log.e("BLEProvisionLanding", "wifiConfigApplyFailed!!")
      }

      override fun provisioningFailedFromDevice(failureReason: ESPConstants.ProvisionFailureReason) {

      }

      override fun deviceProvisioningSuccess() {
        Log.e("BLEProvisionLanding", "deviceProvisioningSuccess!!")
        provisionStatus = "Provisioned!!"
      }

      override fun onProvisioningFailed(e: java.lang.Exception) {
        Log.e("BLEProvisionLanding", "onProvisioningFailed!!")
      }

    })

    return provisionStatus
  }

  private val bleScanListener: BleScanListener = object : BleScanListener {

    override fun scanStartFailed() {
      Log.d("BLEProvisionLanding", "Please turn on Bluetooth to connect BLE device")

//            Toast.makeText(this, "Please turn on Bluetooth to connect BLE device", Toast.LENGTH_SHORT).show()
    }

    override fun onPeripheralFound(device: BluetoothDevice, scanResult: ScanResult) {
      Log.d("BLEProvisionLanding", "BLE device $device")
      Log.d("BLEProvisionLanding", "BLE device $device")

      val dev = BleDevice()
      dev.name = scanResult.scanRecord!!.deviceName.toString()
      if (scanResult.scanRecord!!.serviceUuids != null && scanResult.scanRecord!!.serviceUuids.size > 0) {
        dev.uuid = scanResult.scanRecord!!.serviceUuids[0].toString()
      }
      dev.bluetoothDevice = (device)

      if(!devices.contains(dev))
      {
        devices.add(dev)
      }
      if(selectedDeviceName == dev.name)
      {
        selectedDevice = dev
      }
    }

    override fun scanCompleted() {
      Log.d("BLEProvisionLanding", "Scan completed")

//            isScanning = false
//            updateProgressAndScanBtn()
    }

    override fun onFailure(e: Exception) {
      Log.e("BLEProvisionLanding", e.message!!)
      e.printStackTrace()
    }
  }


  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    this.context = binding.activity
    initilizeWithContext()
  }

  override fun onDetachedFromActivityForConfigChanges() {
    TODO("Not yet implemented")
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    TODO("Not yet implemented")
  }

  override fun onDetachedFromActivity() {
    TODO("Not yet implemented")
  }
}
